/*

libualloc - microlib for memory allocation with various strategies

Copyright (c) 2020 Tibor 'Igor2' Palinkas
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. Neither the name of the Author nor the names of contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

Source code: svn://svn.repo.hu/libualloc/trunk
Contact the author: http://igor2.repo.hu/contact.html

*/

#	include <libualloc/stacks_api.h>
#	include <libualloc/stacks_impl.h>



static long draw_stacks(uall_stacks_t *st)
{
	uall_stacks_page_t *page;
	char *s;
	long cnt = 0;

	printf(" stack:\n");
	for(page = st->pages; page != NULL; page = page->next) {
		printf("  a=%ld u=%ld:", page->alloced, page->used);
		for(s = page->data + page->used - st->esize; s >= page->data; s -= st->esize) {
			long *udt = (long *)s;
			cnt++;
			printf(" %ld", *udt);
		}
		printf("\n");
	}
	return cnt;
}

int main_stacks(int argc, char *argv[])
{
	uall_sysalloc_t sys;
	uall_stacks_t st = {0};
	char *cmd, *arg;
	long *udt;
	int its = 128, n, trace = 0;
	long alloc_cnt = 0, free_cnt = 0, cnt;

	sys.alloc = uall_stdlib_alloc;
	sys.free = uall_stdlib_free;
	sys.page_size = 128;
	st.elem_size = 12;

	for(n = 1; n < argc; n++) {
		cmd = argv[n]; arg = argv[n+1];
		if (cmd == NULL) continue;
		while(*cmd == '-') cmd++;
		switch(*cmd) {
			case 's': rnd_seed = atoi(arg); n++; break;
			case 'i': its = atoi(arg); n++; break;
			case 'p': sys.page_size = atoi(arg); n++; break;
			case 'e': st.elem_size = atoi(arg); n++; break;
			case 't': trace = 1; break;
			default:
				fprintf(stderr, "Unrecognized argument '%s'\n", argv[n]);
				return -1;
		}
	}

	st.sys = &sys;

	/* randomly allocate and free */
	for(n = 0; n < its; n++) {
		if (rnd() % 2) {
			udt = uall_stacks_alloc(&st);
			assert(udt != NULL);
			*udt = alloc_cnt;
			alloc_cnt++;
			if (trace)
				printf("alloc: %ld -> %p\n", *udt, udt);
		}
		else {
			if (trace) {
				udt = uall_stacks_top(&st);
				if (udt != NULL)
					printf("free: %ld -> %p\n", *udt, udt);
			}
			if (uall_stacks_free(&st))
				free_cnt++;
		}
	}

	printf("*** Alloced: %ld freed: %ld\n", alloc_cnt, free_cnt);
	cnt = draw_stacks(&st);

	printf("elems=%ld\n", cnt);
	/* all elems that are not free'd are currently allocated and stored in the pool */
	assert((alloc_cnt - free_cnt) == cnt);

	uall_stacks_clean(&st);

	return 0;
}
